function out=gui2builder(fig,sceFileName)
// Creates an input file to guibuilder from an existing graphical user interface (gui).
//
// Calling Sequence
//   gui2builder(fig,sceFileName)
//
// Parameters
// fig: handle to an existing gui (figure handle)
// sceFileName: optional string with a valid file name where code is to be saved  
//
// Description
// Creates a valid input file to guibuilder from an existing gui.
// 
// A graphical user interface created with for example guimaker can be easily 
// ported to guibuilder for further manual modifications. 
//
// This version of gui2builder does not yet support axes or menus. 
// 
// NOTE: guibuilder can be installed using the command: atomsInstall('guibuilder').
// See http://atoms.scilab.org/toolboxes/guibuilder for more info on the guibuilder toolbox.
// 
// Examples
// page=list(); // Example using guimaker
// page($+1)=list(list([1 10],'frame','General properties'));
// page($+1)=list(list('text','Particle diameter [nm]'),list('edit','1000'));
// page($+1)=list(list('text','Particle density [g/cm3]'),list('edit','1'));
// page($+1)=list(list('text','Gas viscosity (0C) u [N*sec/m2]'),list('edit','0.0000172'));
// page($+1)=list(list('text','Gas density (0C), 1 atm [kg/m3]'),list('edit','1.29'));
// page($+1)=list(list('text','Mean free path (0C), 1 atm [nm]'),list('edit','67'));
// page($+1)=list(list('text','Gamma, Cv/Cp'),list('edit','1.4'));
// page($+1)=list(list('text','Sutherlands temperature, [K]'),list('edit','110.4'));
// page($+1)=list(list('text','Temperature, [K]'),list('edit','300'));
// page($+1)=list(list('text','Volume flow rate [standard l/min]'),list('edit','0.5'));
// page($+1)=list(list([1 4],'frame','Telescope properties'));
// page($+1)=list(list('text','Orifice diameters [mm]'),list('edit','1,1,1,1,1'));
// page($+1)=list(list('text','Tube diameters [mm]'),list('edit','100,100,100,100,100'));
// page($+1)=list(list('text','Input diameters [mm]'),list('edit','201325'));
// page($+1)=list(list(2),list('pushbutton','Stop','callback','OK=%t'),list(2));
// guimaker(page,list('Aerodynamic Lens Design'),[],2); // create a gui with guimaker
// gui2builder(gcf(),'AeroDynLensDes.sce'); // create code for guibuilder 
// guibuilder('AeroDynLensDes.sce')
//
// // Example from one of the Scilab demos.
// exec SCI\modules\gui\demos\uicontrol.dem.sce
// gui2builder(gcf(),'uicontrol.sce'); // create code for guibuilder 
// guibuilder('uicontrol.sce')
//
// See also
//  guibuilder
//  guimaker
//
// Authors
//  T. Pettersen, top@tpett.com
//

if argn(2)==0 then fig=gcf(); end
if argn(2)==1 then sceFileName=''; end

apifun_checktype ( "maker2builder" , fig , "fig" , 1 , "handle" );
apifun_checktype ( "maker2builder" , sceFileName , "sceFileName" , 2 , "string" );

code=[
    '// This GUI file is generated by gui2builder and should work with guibuilder version 2.2'
    '//////////'
    'f=figure(''figure_position'',['+sprintf('%g ',fig.figure_position')+'],''figure_size'',['+..
        sprintf('%g ',fig.figure_size')+'],''auto_resize'',''on'',''figure_name'','''+fig.figure_name+''');';
    '//////////'
    'delmenu(f.figure_id,gettext(''File''))'
    'delmenu(f.figure_id,gettext(''?''))'
    'delmenu(f.figure_id,gettext(''Tools''))'
    'toolbar(f.figure_id,''off'')'
    'handles.dummy = 0;'
];

cbcode=[
    '//////////'
    '// Callbacks are defined as below. Please do not delete the comments as it will be used in coming version'
    '//////////'
];
objs=fig.children;
nobj=length(objs);
uicontrol_props=['Units','BackgroundColor','Enable','FontAngle','FontName','FontSize','FontUnits','FontWeight','ForegroundColor','HorizontalAlignment','ListboxTop','Max','Min','Position','Relief','SliderStep','String','Style','Value','VerticalAlignment','Visible'];
axes_props=['visible','grid','grid_position','x_location','y_location','title','x_label','y_label','z_label','box','filled','sub_ticks','font_style','font_size','font_color','fractional_font','isoview','cube_scaling','view','rotation_angles','log_flags','tight_limits','margins','axes_bounds','auto_clear','auto_scale','hidden_axis_color','line_mode','line_style','thickness','mark_mode','mark_style','mark_style_unit','mark_size','mark_foreground','mark_background','foreground','background','arc_drawing_method','clip_state'];
menu_props=['Enable','Foregroundcolor','Label','Visible','Callback','Callback_type','Checked','Tag'];

TagNo=1; objLine=''; AxTagNo=1;
for i=nobj:-1:1    // for every object 
    obj=objs(i);
    if obj.type=='uicontrol' then
        if isempty(obj.Tag) then    // get Tag identifier for obj
            Tag=sprintf('ObjNo_%i',TagNo);
            TagNo=TagNo+1;
        else
            Tag=obj.Tag;
        end
    end
    select obj.type, 
    case "uicontrol" then
        if isempty(obj.Callback) then
            CallBack=sprintf('%s_callback(handles)',Tag);
        else
            CallBack=obj.Callback;
            CallBack=strsubst(CallBack,'''','''''');
        end
        objLine=sprintf('handles.%s=uicontrol(f,',Tag); 
        objLine=objLine+properties2code(obj,uicontrol_props); 
        objLine=objLine+sprintf('''Tag'',''%s'',',Tag);
        objLine=objLine+sprintf('''Callback'',''%s'')',CallBack); 
        
        code=[code;objLine];
        cbcode=[cbcode;sprintf('function %s_callback(handles)',Tag);..
                       sprintf('// Write your callback for  %s  here)\nendfunction\n\n',Tag)];
    case "Axes" then
        AxTag=sprintf('axTag%i',AxTagNo); AxTagNo+AxTagNo+1;
        objLine=sprintf('handles.%s=newaxes();',AxTag);
        objLine=objLine+sprintf('handles.%s.margins=[',AxTag)+sprintf('%g ',obj.margins(:))+'];';
        objLine=objLine+sprintf('handles.%s.axes_bounds=[',AxTag)+sprintf('%g ',obj.axes_bounds(:))+'];';

// Does not support axes yet.
//        code=[code;objLine];
    end
end
code=[code;cbcode];

if argn(2)>1 then
    if ~isempty(fileinfo(sceFileName)) then
        btn=messagebox([sceFileName+' exists!'],'maker2builder','warning',['Overwrite','Cancel'],'modal');
        if btn==2 then
            warning('maker2builder: Terminated by user.')
            return;
        end
    end
    f=mopen(sceFileName,'w'); 
    mputl(code,f); 
    mclose(f);    
    out=%t;
    if argn(2)<3 then
        editor(sceFileName);
    end
else
    out=code;
end
endfunction
///////////////////////
function objLine=properties2code(obj,props)
// turn obj.property values into Scilab code
objLine='';
for j=1:size(props,'*'),
    objLine=objLine+sprintf('''%s'',',props(j)); 
    Value=get(obj,props(j)); 
    if typeof(Value)=='string' then
        if size(Value,'*')>1 then
            objLine=objLine+'['+sprintf('''%s'' ',Value(:))+'],';
        else
            objLine=objLine+sprintf('''%s'',',Value);
        end
    else
        if isempty(Value) then 
            objLine=objLine+'[0],';
        else
            objLine=objLine+'['+sprintf('%g ',Value(:))+'],';
        end
    end
end
endfunction